﻿########################################################################
#                                                                      #
#  VMware and its employees are not responsible for any issues caused  #
#  by this script. Use at your own discretion and always make sure     #
#  you understand the function and purposes of this script prior to    #
#  running it in your environment.                                     #
#                                                                      #
########################################################################

$WarningPreference = 'SilentlyContinue'
Set-PowerCLIConfiguration -Scope AllUsers -ParticipateInCEIP:$false -Confirm:$false -DisplayDeprecationWarnings:$false | Out-Null
Set-PowerCLIConfiguration -InvalidCertificateAction Ignore -Confirm:$false | Out-Null 

#Menu Function
function Show-Menu { 
    Clear-Host
    Write-Host '
        ___                ____       ____                   
       /   |  ____  ____  / __ \___  / __/__  ____  ________ 
      / /| | / __ \/ __ \/ / / / _ \/ /_/ _ \/ __ \/ ___/ _ \
     / ___ |/ /_/ / /_/ / /_/ /  __/ __/  __/ / / (__  )  __/
    /_/  |_/ .___/ .___/_____/\___/_/  \___/_/ /_/____/\___/ 
          /_/   /_/                                          
       ______                __     ____                
      / ____/_  _____  _____/ /_   / __ \________  ____ 
     / / __/ / / / _ \/ ___/ __/  / /_/ / ___/ _ \/ __ \
    / /_/ / /_/ /  __(__  ) /_   / ____/ /  /  __/ /_/ /
    \____/\__,_/\___/____/\__/  /_/   /_/   \___/ .___/ 
                                               /_/      
    '
    ""
    Write-Host "Press '1' to prepare VMs you manually define" 
    Write-Host "Press '2' to prepare Windows VMs in a specific folder." 
    Write-Host "Press '3' to prepare Windows VMs in a cluster."
    Write-Host "Prest '4' to prepare Windows VMs listed in a file (.csv or .txt)" 
    Write-Host "Press 'Q' to quit." 
    ""
} 

Clear-Host
Write-Host '
    ___                ____       ____                   
   /   |  ____  ____  / __ \___  / __/__  ____  ________ 
  / /| | / __ \/ __ \/ / / / _ \/ /_/ _ \/ __ \/ ___/ _ \
 / ___ |/ /_/ / /_/ / /_/ /  __/ __/  __/ / / (__  )  __/
/_/  |_/ .___/ .___/_____/\___/_/  \___/_/ /_/____/\___/ 
      /_/   /_/                                          
   ______                __     ____                
  / ____/_  _____  _____/ /_   / __ \________  ____ 
 / / __/ / / / _ \/ ___/ __/  / /_/ / ___/ _ \/ __ \
/ /_/ / /_/ /  __(__  ) /_   / ____/ /  /  __/ /_/ /
\____/\__,_/\___/____/\__/  /_/   /_/   \___/ .___/ 
                                           /_/      
'

""
""
Start-Sleep -Seconds 1
Write-Host "Enter the vCenter you wish to connect to:" -ForegroundColor Yellow
$vcenter = Read-Host
$vc_creds = Get-Credential -Message "Please enter the vCenter Admin Credentials"

#Connect to vCenter Server
Try{
    Connect-VIServer $vcenter -Credential $vc_creds -ErrorAction Stop | Out-Null
}
catch{
    Write-Host $Error[0] -ForegroundColor Red
    break
} 

#Do Until loop to go through the menu until expressly told to quit. 
do { 
    Show-Menu 
    $input = Read-Host "Please make a selection" 
    switch($input){ 
        1{
            $defined = @()
            do {
             $definedvms = (Read-Host "Please enter a VM name or enter 'q' when done")
             if ($definedvms -ne 'q') {$defined += $definedvms}
            }
            until ($definedvms -eq 'q')
            $vms = Get-VM $defined -ErrorAction SilentlyContinue
        }
        2{
            $folder = Read-Host "Enter Folder Name"
            $vms = Get-Folder -Name $folder |  Get-VM | Where-Object{$_.Guest.OSFullName -match ('windows')}
        }
        3{
            $cluster = Read-Host "Enter Cluster Name"
            $vms = Get-Cluster $cluster | Get-VM | Where-Object{$_.Guest.OSFullName -match ('windows')}
        }
        4{
            $file = Read-Host "Enter File complete Path"
            $list = Get-Content $file | Select-Object -Skip 1
            $vms = Get-VM $list
        }
        q{
            return 
        } 
    }

    #Create Logs Directory
    $path = "C:\Temp\AppDlogFiles"
    if(!(Test-Path $path)){
        New-Item -ItemType Directory -Path C:\Temp -Name "AppDlogFiles" | Out-Null
    }
        
    #Set Advanced Setting Param
    $param = 'guestIntegrity.enable'

    #Define Desired Setting
    $desiredSetting = @(
        @{
        Name = 'guestIntegrity.enable'
        Value = $true
        }
    )

    Write-Host "Beginning Guest Integrity Configuration" -ForegroundColor Yellow
    ""
    Start-Sleep -Seconds 1
    #Loop to enabled guest integrity in all windows vms
    Try{
        $Correct = @()
        $Incorrect = @()
        $New = @()
        foreach($vm in $vms){
            $desiredSetting | %{
                $setting = Get-AdvancedSetting -Entity $vm -Name $_.Name -ErrorAction Stop
                if($setting){
                    if($setting.Value -eq $_.Value){
                        $CorrectVM = @{Name=$vm}
                        $CorrectObject = New-Object PSObject -Property $CorrectVM
                        $Correct += $CorrectObject
                    }
                    else{
                        $IncorrectVM = @{Name=$vm}
                        $IncorrectObj = New-Object PSObject -Property $IncorrectVM
                        $Incorrect += $IncorrectObj
                        Set-AdvancedSetting -AdvancedSetting $setting -Value $_.Value -Confirm:$false -ErrorAction Stop
                    }
                }
                else{
                    $NewVM = @{Name=$vm}
                    $NewObj = New-Object PSObject -Property $NewVM
                    $New += $NewObj
                    New-AdvancedSetting -Name $_.Name -Value $_.Value -Entity $vm -Confirm:$false -ErrorAction Stop
                }
            }
        }

        Start-Sleep -Seconds 1
        if($Correct.Count -ne 0){
            Write-Host "There were" $Correct.Count "VMs already configured Correctly. No action taken." -ForegroundColor Green
            Start-Sleep -Seconds 1
        }
        if($Incorrect.Count -ne 0){
            Write-Host "There were" $Incorrect.Count "VMs that needed to be corrected" -ForegroundColor Yellow
            Start-Sleep -Seconds 1
        }
        if($New.Count -ne 0){
            Write-Host "There were" $New.Count "VMs that were configured for the first time" -ForegroundColor Yellow
            Start-Sleep -Seconds 1
        }
        if(($New + $Incorrect).Count -ne 0){
            Out-File -FilePath C:\Temp\AppDlogFiles\'vms_enabled_with_guest_integrity_.txt' -InputObject $($New + $Incorrect) -Force 
        }
    }
    catch{
        Write-Host "There was an error in the VM ""$vm""" $Error[0] -ForegroundColor Red
        break
    }
    
    ""
    Write-Host "Starting AppDefense Installation. Please enter the Windows VM Admin Credentials" -ForegroundColor Yellow
    Start-Sleep -Seconds 2
    $os_creds = Get-Credential -Message "Enter the Windows VM Guest OS Admin Credentials"
    $alreadyInstalled = @()
    $installed = @()
    $poweredoff = @()
    $failedvm = @()
    $i = 0
    foreach($vm in $vms){
    $i++
    Write-Progress -activity "Configuring AppDefense in $vm. This will take some time..." -status "Configuring: $i of $($vms.Count)" -percentComplete (($i / $vms.Count)  * 100)
        if($vm.PowerState -eq "PoweredOn"){
            $QueryScriptText = 'sc query glxgi'
            $QueryResult = (Invoke-VMScript -VM $vm -ScriptText $QueryScriptText -ScriptType bat -GuestCredential $os_creds -ErrorAction SilentlyContinue -ErrorVariable LoginFailure).ScriptOutput
            if($LoginFailure){
                $LoginFailedvm = @{Name=$vm}
                $LoginFailedObj = New-Object PSObject -Property $LoginFailedvm
                $LoginFailed = $LoginFailedObj
                Out-File -FilePath C:\Temp\AppDlogFiles\'login_failed_vms.txt' -InputObject $LoginFailed -Force
                Continue
            }
            if($QueryResult.Contains("RUNNING")){
                $alreadyInstalledvm = @{Name=$vm}
                $alreadyInstalledObj = New-Object PSObject -Property $alreadyInstalledvm
                $alreadyInstalled = $alreadyInstalledObj
                Out-File -FilePath C:\Temp\AppDlogFiles\'vms_already_installed.txt' -InputObject $alreadyInstalled -Force
            }
            else{
                #Install AppDefense in VM
                $getproductcode = '(Get-WmiObject -Class Win32_Product | Where-Object {$_.Name -match "VMware Tools"}).IdentifyingNumber'
                $productcode = (Invoke-VMScript -VM $vm -ScriptText $getproductcode -ScriptType Powershell -GuestCredential $os_creds -ErrorAction Inquire).ScriptOutput.Trim()
                $InstallScriptText = "msiexec /i $productcode /qn /quiet /norestart ADDLOCAL=AppDefense"
                do {
                    $InstallResult = (Invoke-VMScript -VM $vm -ScriptText $InstallScriptText -ScriptType bat -GuestCredential $os_creds -ErrorAction Inquire).ExitCode
                }
                until ($InstallResult = "0" -or "3010")
                    
                    
                #Write successfull installation to log file to reboot. 
                $ExitCode = $InstallResult.ExitCode
                if ($ExitCode = "3010"){
                    #Add each installed VM to the variable $installed.
                    $installedvm = @{Name=$vm}
                    $installedObj = New-Object PSObject -Property $installedvm
                    $installed += $installedObj
                    Out-File -FilePath C:\Temp\AppDlogFiles\'vms_to_powercycle.txt' -InputObject $installed -Force
                }

                #Write failed installation to failed VMs log file. 
                else {
                    $failedvm = @{Name=$vm}
                    $failedObj = New-Object PSObject -Property $failedvm
                    $failed += $failedObj
                    Out-File -FilePath C:\Temp\AppDlogFiles\'failed_vms.txt' -InputObject $failed -Force
                }
            }
        }
        else{
            #Add VM to powered off VM log file and variable. 
            Start-Sleep -Seconds 2
            ""
            $poweredoffvm = @{Name=$vm}
            $poweredoffobj = New-Object PSObject -Property $poweredoffvm
            $poweredoff += $poweredoffobj
            Out-File -FilePath C:\Temp\AppDlogFiles\'poweredoff_vms.txt' -InputObject $poweredoff -Force
            Start-Sleep -Seconds 2
        }
    }
    Write-Progress -Activity "Configuring AppDefense. This will take some time. . ." -Status "Ready" -Completed


    Clear
    Start-Sleep -Seconds 1
    Write-Host "Script Complete" -ForegroundColor Green

    if($installed.Count -ne 0){
        ""
        Write-Host "AppDefense installed in" $installed.Count "VM(s)" -ForegroundColor Green
        Start-Sleep -Seconds 1
    }
    if(($Incorrect + $New).Count -ne 0){
        ""
        Write-Host "Guest Integrity enabled in" ($incorrect + $New).Count "VM(s)" -ForegroundColor Green
        Start-Sleep -Seconds 1
    }
    if($poweredoff.Count -ne 0){
        ""
        Write-Host "AppDefense could not be installed in" $poweredoff.Count "VM(s) because they were powered off" -ForegroundColor Red
        Write-Host "See the below log file for the VM(s)"
        Write-Host "'C:\Temp\AppDlogFiles\poweredoff_vms.txt'"
        Start-Sleep -Seconds 1
    }
    if(($installed + $Incorrect + $New + $poweredoff).Count -eq 0){
        ""
        Write-Host "No VMs needed to be configured for AppDefense"
        Start-Sleep -Seconds 1
    }
    ""
    if($installed.Count -ne 0 -and ($Incorrect + $New).Count -ne 0){
        ""
        Write-Host "***IMPORTANT***" -ForegroundColor Yellow
        Write-Host "For the changes made by this script to take effect, you must completely shutdown and startup the VM's found in the below log files" -ForegroundColor Yellow
        Write-Host "'C:\Temp\AppDLogFiles\vms_to_powercycle.txt'"
        Write-Host "'C:\Temp\AppDLogFiles\vms_enabled_with_guest_integrity_.txt'"
        Start-Sleep -Seconds 2
    }
    if($installed.Count -ne 0){
        ""
        Write-Host "***IMPORTANT***" -ForegroundColor Yellow
        Write-Host "For the changes made by this script to take effect, you must completely shutdown and startup the VM's found in the below log files" -ForegroundColor Yellow
        Write-Host "'C:\Temp\AppDLogFiles\vms_to_powercycle.txt'" -ForegroundColor Yellow
        Start-Sleep -Seconds 2
    }
    if($alreadyInstalled.Count -ne 0){
        ""
        Write-Host "No further action needed on" $alreadyInstalled.Name.Count "VM(s) because AppDefense was already properly configured on the VMs selected" -ForegroundColor Green
        Write-Host "Please review C:\Temp\AppDlogFiles\'vms_already_installed.txt' for a list of VMs already configured for AppDefense." -ForegroundColor Green
        Start-Sleep -Seconds 1
    }

    if($LoginFailed.Count -ne 0){
        ""
        Write-Host "***ERRORS***" -ForegroundColor Red
        Write-Host "You were unable to authenticate to the some VMs." -ForegroundColor Red
        Write-Host "Please review C:\Temp\AppDlogFiles\'login_failed_vms.txt' for a list of VMs you weren't able to authenticate to" -ForegroundColor Red
        Start-Sleep -Seconds 1
    }
    ""
    Read-Host "Please press ""Enter"" to return to the Menu" 
} 
until ($input -eq 'q') 

